@@ -1,16 +1,14 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond import backend
 from trytond.i18n import gettext
 from trytond.model import ModelSQL, ValueMixin, fields
 from trytond.model.exceptions import RequiredValidationError
-from trytond.pyson import Eval
-from trytond.pool import Pool, PoolMeta
-from trytond.tools.multivalue import migrate_property
-
 from trytond.modules.product import round_price
 from trytond.modules.purchase.purchase import (
     get_shipments_returns, search_shipments_returns)
+from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Eval
+from trytond.transaction import Transaction
 
 purchase_drop_location = fields.Many2One(
     'stock.location', "Purchase Drop Location", domain=[('type', '=', 'drop')])
@@ -26,14 +24,13 @@ class Request(metaclass=PoolMeta):
         context={
             'company': Eval('company', -1),
             },
-        depends=['company'])
+        depends={'company'})
     delivery_address = fields.Many2One('party.address', 'Delivery Address',
         domain=[('party', '=', Eval('customer'))],
         states={
             'invisible': ~Eval('customer'),
             'readonly': Eval('state') != 'draft',
-            },
-        depends=['customer', 'state'])
+            })
 
 
 class Configuration(metaclass=PoolMeta):
@@ -52,23 +49,6 @@ class ConfigurationPurchaseDropLocation(ModelSQL, ValueMixin):
     __name__ = 'purchase.configuration.purchase_drop_location'
     purchase_drop_location = purchase_drop_location
 
-    @classmethod
-    def __register__(cls, module_name):
-        exist = backend.TableHandler.table_exist(cls._table)
-
-        super().__register__(module_name)
-
-        if not exist:
-            cls._migrate_property([], [], [])
-
-    @classmethod
-    def _migrate_property(cls, field_names, value_names, fields):
-        field_names.append('purchase_drop_location')
-        value_names.append('purchase_drop_location')
-        migrate_property(
-            'purchase.configuration', field_names, cls, value_names,
-            fields=fields)
-
     @classmethod
     def default_purchase_drop_location(cls):
         pool = Pool()
@@ -90,28 +70,25 @@ class Purchase(metaclass=PoolMeta):
         context={
             'company': Eval('company', -1),
             },
-        depends=['company'])
+        depends={'company'})
     delivery_address = fields.Many2One('party.address', 'Delivery Address',
         domain=[('party', '=', Eval('customer'))],
         states={
             'readonly': Eval('state') != 'draft',
             'invisible': ~Eval('customer'),
-            },
-        depends=['state', 'customer'])
+            })
     drop_shipments = fields.Function(fields.Many2Many('stock.shipment.drop',
             None, None, "Drop Shipments",
             states={
                 'invisible': ~Eval('customer'),
-                },
-            depends=['customer']),
+                }),
         'get_drop_shipments', searcher='search_drop_shipments')
     drop_location = fields.Many2One('stock.location', 'Drop Location',
         domain=[('type', '=', 'drop')],
         states={
             'invisible': ~Eval('customer', False),
             'required': Eval('customer', False),
-            },
-        depends=['customer'])
+            })
 
     @staticmethod
     def default_drop_location():
@@ -147,18 +124,22 @@ class Purchase(metaclass=PoolMeta):
                     'purchase': self.rec_name,
                     })
 
-    def create_move(self, move_type):
+    @classmethod
+    def _process_shipment(cls, purchases):
         pool = Pool()
         DropShipment = pool.get('stock.shipment.drop')
 
-        moves = super(Purchase, self).create_move(move_type)
-        if moves and self.customer and move_type == 'in':
-            drop_shipment = self.create_drop_shipment()
-            drop_shipment.supplier_moves = moves
-            drop_shipment.save()
-            DropShipment.wait([drop_shipment])
-
-        return moves
+        drop_shipments = []
+        for purchase in purchases:
+            if purchase.customer:
+                moves = purchase.create_move('in')
+                if moves:
+                    drop_shipment = purchase.create_drop_shipment()
+                    drop_shipment.supplier_moves = moves
+                    drop_shipments.append(drop_shipment)
+        DropShipment.save(drop_shipments)
+        DropShipment.wait(drop_shipments)
+        super()._process_shipment(purchases)
 
     def create_drop_shipment(self):
         pool = Pool()
@@ -199,8 +180,7 @@ class ProductSupplier(metaclass=PoolMeta):
     drop_shipment = fields.Boolean('Drop Shipment',
         states={
             'invisible': ~Eval('drop_shipment_available', False),
-            },
-        depends=['drop_shipment_available'])
+            })
     drop_shipment_available = fields.Function(
         fields.Boolean("Drop Shipment Available"),
         'on_change_with_drop_shipment_available')
@@ -211,7 +191,7 @@ class ProductSupplier(metaclass=PoolMeta):
     def on_change_with_drop_shipment_available(self, name=None):
         product = self.product or self.template
         if product and product.type in {'goods', 'assets'}:
-            return product.supply_on_sale
+            return bool(product.supply_on_sale)
 
 
 class RequestCreatePurchase(metaclass=PoolMeta):
@@ -235,6 +215,8 @@ class HandleShipmentException(metaclass=PoolMeta):
         pool = Pool()
         Sale = pool.get('sale.sale')
         Move = pool.get('stock.move')
+        transaction = Transaction()
+        context = transaction.context
 
         super().transition_handle()
 
@@ -257,5 +239,7 @@ class HandleShipmentException(metaclass=PoolMeta):
         if moves:
             Move.cancel(moves)
         if sales:
-            Sale.__queue__.process(sales)
+            with transaction.set_context(
+                    queue_batch=context.get('queue_batch', True)):
+                Sale.__queue__.process(sales)
         return 'end'
