@@ -1,14 +1,10 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond import backend
 from trytond.model import ModelSQL, ValueMixin, fields
-from trytond.pyson import Eval
-from trytond.pool import Pool, PoolMeta
-from trytond.transaction import Transaction
-from trytond.tools.multivalue import migrate_property
-
 from trytond.modules.sale.sale import (
     get_shipments_returns, search_shipments_returns)
+from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Eval
 
 sale_drop_location = fields.Many2One(
     'stock.location', "Sale Drop Location", domain=[('type', '=', 'drop')])
@@ -30,23 +26,6 @@ class ConfigurationSaleDropLocation(ModelSQL, ValueMixin):
     __name__ = 'sale.configuration.sale_drop_location'
     sale_drop_location = sale_drop_location
 
-    @classmethod
-    def __register__(cls, module_name):
-        exist = backend.TableHandler.table_exist(cls._table)
-
-        super().__register__(module_name)
-
-        if not exist:
-            cls._migrate_property([], [], [])
-
-    @classmethod
-    def _migrate_property(cls, field_names, value_names, fields):
-        field_names.append('sale_drop_location')
-        value_names.append('sale_drop_location')
-        migrate_property(
-            'sale.configuration', field_names, cls, value_names,
-            fields=fields)
-
     @classmethod
     def default_sale_drop_location(cls):
         pool = Pool()
@@ -82,21 +61,21 @@ class Sale(metaclass=PoolMeta):
     get_drop_shipments = get_shipments_returns('stock.shipment.drop')
     search_drop_shipments = search_shipments_returns('stock.shipment.drop')
 
-    def create_shipment(self, shipment_type):
-        shipments = super(Sale, self).create_shipment(shipment_type)
-        if shipment_type == 'out':
-            with Transaction().set_context(_drop_shipment=True):
-                self.create_drop_shipment_moves()
-        return shipments
-
-    def create_drop_shipment_moves(self):
+    @classmethod
+    def _process_supply(cls, sales, product_quantities):
         pool = Pool()
         Move = pool.get('stock.move')
+        super()._process_supply(sales, product_quantities)
+        moves = []
+        for sale in sales:
+            moves.extend(sale.create_drop_shipment_moves())
+        Move.save(moves)
 
+    def create_drop_shipment_moves(self):
         moves = []
         for line in self.lines:
             moves += line.get_drop_moves()
-        Move.save(moves)
+        return moves
 
 
 class Line(metaclass=PoolMeta):
@@ -120,20 +99,18 @@ class Line(metaclass=PoolMeta):
             return False
         if self.purchase_request:
             purchase_line = self.purchase_request.purchase_line
-            if purchase_line and purchase_line.move_done:
+            if purchase_line and purchase_line.moves_progress >= 1:
                 return False
         return True
 
     def get_move(self, shipment_type):
-        result = super().get_move(shipment_type)
-        if (shipment_type == 'out'
-                and not Transaction().context.get('_drop_shipment')):
-            if self.supply_on_sale_drop_move:
-                return
-        return result
+        move = super().get_move(shipment_type)
+        if self.supply_on_sale_drop_move and not self.purchase_request:
+            move = None
+        return move
 
-    def get_purchase_request(self):
-        request = super().get_purchase_request()
+    def get_purchase_request(self, product_quantities):
+        request = super().get_purchase_request(product_quantities)
         if request and request.party:
             if self.product and self.product.type in ('goods', 'assets'):
                 product_supplier = request.find_best_product_supplier(
